package cn.com.ctop.kuaishou.modules.report.service.impl;

import cn.com.ctop.common.module.entity.RuleDataTargetKuaishou;
import cn.com.ctop.common.module.entity.RuleDatePlanKuaishou;
import cn.com.ctop.common.module.entity.RuleDateUnitKuaishou;
import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.common.module.service.IRuleDataTargetKuaishouService;
import cn.com.ctop.common.module.service.IRuleDataUnitKuaishouService;
import cn.com.ctop.common.module.service.IRuleDatePlanKuaishouService;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.common.module.utils.HttpUtils;
import cn.com.ctop.kuaishou.modules.batch.entity.KuaiShouGroup;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaiShouGroupService;
import cn.com.ctop.kuaishou.modules.report.service.IRuleKuaiShouPlanService;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxl.job.core.context.XxlJobHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class RuleKuaiShouPlanServiceImpl implements IRuleKuaiShouPlanService {

    @Autowired
    ICtopOauthTokenService oauthTokenService;

    @Autowired
    IRuleDataTargetKuaishouService targetKuaishouService;

    @Autowired
    IRuleDatePlanKuaishouService planKuaishouService;

    @Autowired
    IRuleDataUnitKuaishouService unitKuaishouService;

    @Autowired
    IKuaiShouGroupService kuaiShouGroupService;

    @Override
    public void cleanRuleDataPlan(Long accountId, int page) {
        JSONObject param = new JSONObject();
        param.put("advertiser_id", accountId);
        param.put("page_size", 500);
        param.put("page", page);

        Map<String, String> headers = new HashMap<>();
        headers.put("Access-Token", oauthTokenService.getTokenByAccountId(accountId).getAccessToken());
        headers.put("Content-Type", "application/json");

        String result = HttpUtils.kuaiShouhttpPostRequest("https://ad.e.kuaishou.com/rest/openapi/v1/campaign/list", param.toJSONString(), headers);
        JSONObject resultJson = JSONObject.parseObject(result);
        if (Check.isNull(resultJson)) {
            XxlJobHelper.log("获取广告计划接口异常,advertiserId:{}", accountId);
            return;
        }
        Integer code = resultJson.getInteger("code");
        if (null == code || code != 0) {
            XxlJobHelper.log("获取广告计划返回结果异常,advertiserId:{},异常信息:{}", accountId, resultJson);
            return;
        }
        JSONArray details = resultJson.getJSONObject("data").getJSONArray("details");
        if (Check.isNull(details)) {
            return;
        }
        this.insertPlan(accountId, details);
        cleanRuleDataPlan(accountId, page + 1);
    }

    @Override
    public void cleanRuleDataUnit(Long accountId, String startDate, String endDate, int page) {
        JSONObject param = new JSONObject();
        param.put("advertiser_id", accountId);
        param.put("page_size", 500);
        param.put("page", page);
        param.put("start_date",startDate);
        param.put("end_date",endDate);

        Map<String, String> headers = new HashMap<>();
        headers.put("Access-Token", oauthTokenService.getTokenByAccountId(accountId).getAccessToken());
        headers.put("Content-Type", "application/json");

        String result = HttpUtils.kuaiShouhttpPostRequest("https://ad.e.kuaishou.com/rest/openapi/v1/report/unit_report", param.toJSONString(), headers);
        JSONObject resultJson = JSONObject.parseObject(result);
        if (Check.isNull(resultJson)) {
            XxlJobHelper.log("获取广告组报表接口异常,advertiserId:{}", accountId);
            return;
        }
        Integer code = resultJson.getInteger("code");
        if (null == code || code != 0) {
            XxlJobHelper.log("获取广告组报表返回结果异常,advertiserId:{},异常信息:{}", accountId, resultJson);
            return;
        }
        JSONArray details = resultJson.getJSONObject("data").getJSONArray("details");
        if (Check.isNull(details)) {
            return;
        }
        this.insertUnit(accountId, details);
        cleanRuleDataUnit(accountId, startDate, endDate, page + 1);
    }

    @Override
    public void cleanRuleDataTarget(Long accountId, int page) {
        JSONObject param = new JSONObject();
        param.put("advertiser_id", accountId);
        param.put("page_size", 500);
        param.put("page", page);

        Map<String, String> headers = new HashMap<>();
        headers.put("Access-Token", oauthTokenService.getTokenByAccountId(accountId).getAccessToken());
        headers.put("Content-Type", "application/json");

        String result = HttpUtils.kuaiShouhttpPostRequest("https://ad.e.kuaishou.com/rest/openapi/v1/ad_unit/list", param.toJSONString(), headers);
        JSONObject resultJson = JSONObject.parseObject(result);
        if (Check.isNull(resultJson)) {
            XxlJobHelper.log("获取广告组接口异常,advertiserId:{}", accountId);
            return;
        }
        Integer code = resultJson.getInteger("code");
        if (null == code || code != 0) {
            XxlJobHelper.log("获取广告组返回结果异常,advertiserId:{},异常信息:{}", accountId, resultJson);
            return;
        }
        JSONArray details = resultJson.getJSONObject("data").getJSONArray("details");
        if (Check.isNull(details)) {
            return;
        }
        this.insertTarget(accountId, details);
        cleanRuleDataTarget(accountId, page + 1);
    }

    private void insertPlan(Long accountId, JSONArray details) {
        List<RuleDatePlanKuaishou> batch = new ArrayList<>();
        details.forEach(detail -> {
            JSONObject planDetail = (JSONObject) detail;
            RuleDatePlanKuaishou plan = new RuleDatePlanKuaishou();
            if (planDetail.getInteger("status") == 4 && planDetail.getInteger("put_status") == 1) {
                plan.setId(planDetail.getLong("campaign_id"));
                plan.setAccountId(accountId);
                plan.setPlanId(planDetail.getLong("campaign_id"));
                plan.setPlanName(planDetail.getString("campaign_name"));
                batch.add(plan);
            }
        });
        if(!batch.isEmpty()) {
            planKuaishouService.saveOrUpdateBatch(batch);
        }
    }

    private void insertUnit(Long accountId, JSONArray details) {
        List<RuleDateUnitKuaishou> batch = new ArrayList<>();
        details.forEach(detail -> {
            JSONObject unitDetail = (JSONObject) detail;
            RuleDateUnitKuaishou unit = new RuleDateUnitKuaishou();
            QueryWrapper<KuaiShouGroup> wrapper=new QueryWrapper<>();
            wrapper.eq("unit_id",unitDetail.getLong("unit_id"));
            KuaiShouGroup group = kuaiShouGroupService.getOne(wrapper);
            if(group!=null){
                unit.setId(unitDetail.getLong("unit_id"));
                unit.setAccountId(accountId);
                unit.setPlanId(unitDetail.getLong("campaign_id"));
                unit.setUnitId(unitDetail.getLong("unit_id"));
                unit.setUnitName(unitDetail.getString("unit_name"));
                unit.setCharge(unitDetail.getBigDecimal("charge"));
                //未知优化目标数据存空
                if(group.getOcpxActionType()!=0){
                    unit.setConvertNum(handleConvertNum(group.getOcpxActionType(),unitDetail));
                    if(unit.getConvertNum()!=0){
                        unit.setConvertCost(unitDetail.getBigDecimal("charge").divide(BigDecimal.valueOf(unit.getConvertNum()), 2, RoundingMode.HALF_UP));
                    }
                }
                batch.add(unit);
            }

        });
        if(!batch.isEmpty()){
            unitKuaishouService.saveOrUpdateBatch(batch);
        }
    }

    private void insertTarget(Long accountId, JSONArray details) {
        List<RuleDataTargetKuaishou> batch = new ArrayList<>();
        details.forEach(detail -> {
            JSONObject unitDetail = (JSONObject) detail;
            RuleDataTargetKuaishou target = new RuleDataTargetKuaishou();
            if ((unitDetail.getInteger("status") == 20 || unitDetail.getInteger("status") == 19) &&
                    unitDetail.getInteger("put_status") == 1) {
                target.setId(unitDetail.getLong("unit_id"));
                target.setAccountId(accountId);
                target.setPlanId(unitDetail.getLong("campaign_id"));
                target.setUnitId(unitDetail.getLong("unit_id"));
                target.setSceneId(unitDetail.getJSONArray("scene_id")==null?"":unitDetail.getJSONArray("scene_id").toString());
                target.setRegion(unitDetail.getJSONArray("region")==null?"":unitDetail.getJSONArray("region").toString());
                target.setGender(unitDetail.getInteger("gender")==null?0:unitDetail.getInteger("gender"));
                target.setPlatformOs(unitDetail.getInteger("platform_os")==null?0:unitDetail.getInteger("platform_os"));
                target.setDeviceBrand(unitDetail.getJSONArray("device_brand")==null?"":unitDetail.getJSONArray("device_brand").toString());
                target.setDevicePrice(unitDetail.getJSONArray("device_price")==null?"":unitDetail.getJSONArray("device_price").toString());
                target.setFilterConvertedLevel(unitDetail.getInteger("filter_converted_level")==null?0:unitDetail.getInteger("filter_converted_level"));
                target.setIsOpen(unitDetail.getInteger("is_open")==null?0:unitDetail.getInteger("is_open"));
                target.setScheduleTime(unitDetail.getString("schedule_time")==null?"":unitDetail.getString("schedule_time"));
                batch.add(target);
            }
        });
        if (!batch.isEmpty()) {
            targetKuaishouService.saveOrUpdateBatch(batch);
        }
    }

    private Long handleConvertNum(int ocpxActionType,JSONObject detail){
        Long convertNum=null;
        switch (ocpxActionType){
            case 2:
                convertNum=detail.getLong("submit");
                break;
            case 10:
                convertNum=detail.getLong("show");
                break;
            case 11:
                convertNum=detail.getLong("photo_click");
                break;
            case 31:
                convertNum=detail.getLong("download_completed");
                break;
            case 53:
                convertNum=detail.getLong("event_valid_clues");
                break;
            case 180:
                convertNum=detail.getLong("activation");
                break;
            case 190:
                convertNum=detail.getLong("event_pay");
                break;
            case 191:
                convertNum=detail.getDouble("event_pay_first_day_roi").longValue();
                break;
            case 348:
                convertNum=detail.getLong("event_valid_clues");
                break;
            case 383:
                convertNum=detail.getLong("event_credit_grant_landing_page");
                break;
            case 384:
                convertNum=detail.getLong("event_jin_jian_landing_page");
                break;
            case 715:
                convertNum=detail.getLong("event_add_wechat");
                break;
        }
        return convertNum;
    }
}
